@@ -94,6 +94,8 @@ enum flags
     HELP_IO,
     HELP_MP4,
     HELP_ADVANCED,
+    OPT_JOINT,
+    OPT_PNS
 };
 
 typedef struct {
@@ -183,7 +185,10 @@ static help_t help_mp4[] = {
 static help_t help_advanced[] = {
     {"--tns  \tEnable coding of TNS, temporal noise shaping.\n"},
     {"--no-tns\tDisable coding of TNS, temporal noise shaping.\n"},
-    {"--no-midside\tDon\'t use mid/side coding.\n"},
+    {"--joint 0\tDisable joint stereo coding.\n"},
+    {"--joint 1\tUse Mid/Side coding.\n"},
+    {"--joint 2\tUse Intensity Stereo coding.\n"},
+    {"--pns <0 .. 10>\tPNS level; 0=disabled.\n"},
     {"--mpeg-vers X\tForce AAC MPEG version, X can be 2 or 4\n"},
     {"--shortctl X\tEnforce block type (0 = both (default); 1 = no short; 2 = no\n"
     "\t\tlong).\n"},
@@ -421,7 +426,8 @@ int main(int argc, char *argv[])
     faacEncConfigurationPtr myFormat;
     unsigned int mpegVersion = MPEG2;
     unsigned int objectType = LOW;
-    static int useMidSide = 1;
+    int jointmode = -1;
+    int pnslevel = -1;
     static int useTns = 0;
     enum container_format container = NO_CONTAINER;
     enum stream_format stream = ADTS_STREAM;
@@ -505,7 +511,8 @@ int main(int argc, char *argv[])
             {"help-mp4", 0, 0, HELP_MP4},
             {"help-advanced", 0, 0, HELP_ADVANCED},
             {"raw", 0, 0, 'r'},
-            {"no-midside", 0, &useMidSide, 0},
+            {"joint", required_argument, 0, OPT_JOINT},
+            {"pns", required_argument, 0, OPT_PNS},
             {"cutoff", 1, 0, 'c'},
             {"quality", 1, 0, 'q'},
             {"pcmraw", 0, 0, 'P'},
@@ -749,6 +756,12 @@ int main(int argc, char *argv[])
             help(c);
             return 1;
             break;
+        case OPT_JOINT:
+            jointmode = atoi(optarg);
+            break;
+        case OPT_PNS:
+            pnslevel = atoi(optarg);
+            break;
         case '?':
         default:
             help('?');
@@ -885,7 +898,10 @@ int main(int argc, char *argv[])
     }
     if (infile->channels >= 6)
         myFormat->useLfe = 1;
-    myFormat->allowMidside = useMidSide;
+    if (jointmode >= 0)
+        myFormat->jointmode = jointmode;
+    if (pnslevel >= 0)
+        myFormat->pnslevel = pnslevel;
     if (quantqual > 0)
     {
         myFormat->quantqual = quantqual;
@@ -952,6 +968,8 @@ int main(int argc, char *argv[])
     else
         fprintf(stderr, "Quantization quality: %ld\n", quantqual);
     fprintf(stderr, "Bandwidth: %d Hz\n", cutOff);
+    if (myFormat->pnslevel > 0)
+        fprintf(stderr, "PNS level: %d\n", myFormat->pnslevel);
     fprintf(stderr, "Object type: ");
     switch (objectType)
     {
@@ -968,8 +986,17 @@ int main(int argc, char *argv[])
     fprintf(stderr, "(MPEG-%d)", (mpegVersion == MPEG4) ? 4 : 2);
     if (myFormat->useTns)
         fprintf(stderr, " + TNS");
-    if (myFormat->allowMidside)
+
+    switch(myFormat->jointmode) {
+    case JOINT_MS:
         fprintf(stderr, " + M/S");
+        break;
+    case JOINT_IS:
+        fprintf(stderr, " + IS");
+        break;
+    }
+    if (myFormat->pnslevel > 0)
+        fprintf(stderr, " + PNS");
     fprintf(stderr, "\n");
 
     fprintf(stderr, "Container format: ");
